#!/usr/bin/python
import chi2_regular_binning, chi2_adaptive_binning
from chi2_plots import *
import os

def example_call():
	orig_name="gaussian_same_projection_redefined_p_value_distribution__0_1__0_075_miranda"
	dim_list = [2]
	comp_file_list_list = []
	adaptive_binning=True
	for dim_data in dim_list:
		comp_file_list=[]
		for i in range(100):
			comp_file_list.append((os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/gaussian_same_projection_on_each_axis/gauss_data/gaussian_same_projection_on_each_axis_redefined_{1}D_1000_0.6_0.2_0.1_{0}.txt".format(i,dim_data),os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/gaussian_same_projection_on_each_axis/gauss_data/gaussian_same_projection_on_each_axis_redefined_{1}D_1000_0.6_0.2_0.075_{0}.txt".format(i,dim_data)))
		comp_file_list_list.append(comp_file_list)
	if adaptive_binning==True:
		number_of_splits_list = [1,2]
		chi2_adaptive_binning.chi2_adaptive_binning(orig_name, dim_list, comp_file_list_list,number_of_splits_list)
	else:
		single_no_bins_list=[2,3,5]
		chi2_regular_binning.chi2_regular_binning(orig_name, dim_list, comp_file_list_list,single_no_bins_list)	





from __future__ import print_function
import sys 
from chi2_plots import *
import random
import ast

"""
This script can be used to get the p value for the Miranda method (=chi squared). It takes input files with column vectors corresponding to 
features and lables. 
"""

print(__doc__)
import sys
sys.path.insert(0,'../..')
import os
from scipy import stats
import numpy as np
import matplotlib.pyplot as plt 
import numpy.matlib
from matplotlib.colors import Normalize

from sklearn.preprocessing import StandardScaler

##############################################################################
# Setting parameters
#
#orig_name= sys.argv[1]
#number_of_splits_list= ast.literal_eval(sys.argv[2])
#print("number_of_splits_list : ", number_of_splits_list)
#dim_list = ast.literal_eval(sys.argv[3])
#comp_file_list_list = ast.literal_eval(sys.argv[4])


def chi2_adaptive_binning_systematics(orig_name, dim_list, comp_file_list_list,number_of_splits_list,systematics_fraction):
	sample1_name="original"
	sample2_name="modified"

	shuffling_seed = 100

	DEBUG = False


	##############################################################################
	max_number_of_splits = np.max(number_of_splits_list)
	for dim_index, dim_data in enumerate(dim_list):
		print("We are now in "+str(dim_data) + " Dimensions")
		#comp_file_list=[]
		comp_file_list = comp_file_list_list[dim_index]
		#for i in range(2):

			#comp_file_list.append((os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/higher_dimensional_gauss/gauss_data/data_high" +str(dim_data)+"Dgauss_10000_0.5_0.1_0.0_{0}.txt".format(i),os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/higher_dimensional_gauss/gauss_data/data_high"+str(dim_data)+"Dgauss_10000_0.5_0.1_0.01_{0}.txt".format(i))) 
			#comp_file_list.append((os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/legendre/legendre_data/data_legendre_contrib0__1__10__sample_{0}.txt".format(i),os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/legendre/legendre_data/data_legendre_contrib0__1__9__sample_{0}.txt".format(i)))
			#comp_file_list.append((os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/gaussian_same_projection_on_each_axis/gauss_data/gaussian_same_projection_on_each_axis_redefined_{1}D_1000_0.6_0.2_0.1_{0}.txt".format(i,dim_data),os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/gaussian_same_projection_on_each_axis/gauss_data/gaussian_same_projection_on_each_axis_redefined_{1}D_1000_0.6_0.2_0.075_{0}.txt".format(i,dim_data)))
		#print(comp_file_list)

		score_dict = {}
		for number_of_splits in number_of_splits_list:
			score_dict[str(number_of_splits)]=[]
		

		for comp_file_0,comp_file_1 in comp_file_list:
			print("Operating of files :"+comp_file_0+"   "+comp_file_1)

			#extracts data from the files
			features_0=np.loadtxt(comp_file_0,dtype='d')
			features_1=np.loadtxt(comp_file_1,dtype='d')

			#determine how many data points are in each sample
			no_0=features_0.shape[0]
			no_1=features_1.shape[0]

			#Give all samples in file 0 the label 0 and in file 1 the feature 1
			label_0=np.zeros((no_0,1))
			label_1=np.ones((no_1,1))

			#Create an array containing samples and features.
			data_0=np.c_[features_0,label_0]
			data_1=np.c_[features_1,label_1]

			data=np.r_[data_0,data_1]

			no_dim = data.shape[1]-1
			assert (no_dim ==dim_data), 	"The dimension of the data is inconsistent {} vs {}".format(no_dim,dim_data)

			np.random.shuffle(data)

			#print("data.shape : ", data.shape)
			labels=data[:,-1]

			X_values= data[:,:-1]
			X_max   = np.amax(data,axis=0)[:-1] 
			X_min   = np.amin(data,axis=0)[:-1]
			X_total_width = (np.subtract(X_max,X_min))
			del data

			#Scaling	
			X_values = X_values - X_min[None,:]
			X_values = X_values / X_total_width[None,:]
			
			data = np.concatenate((X_values, labels[:,None]), axis=1)

			#print("X_values : ",X_values)
			starting_boundary = []
			for i in range(dim_data):
				starting_boundary.append([0.0,1.0])
			#Each key has the following stricture: # of splits and for each split if it was closer (a) or further away from (b) the origin. The original bin is "0"
			#For example "2ab" means it is the bin that was closer to the origin for the first split and further away for the second one.
			bin_boundaries_dict = {'0' : np.array(starting_boundary)}
			bin_points_dict = {'0' : data}

			for split_number in range(1,1+max_number_of_splits):
				for bin_key, bin_boundary in bin_boundaries_dict.items():
					if str(split_number-1) in bin_key:
						variances= np.var(bin_points_dict[bin_key][:,:-1], axis=0)
						#print("\nvariances : ", variances)
						dim_to_be_sliced = np.argmax(variances)
						#print("dim_to_be_sliced : ",dim_to_be_sliced)
						#print("bin_points_dict[bin_key] : ",bin_points_dict[bin_key])
						#print("bin_points_dict[bin_key][:,dim_to_be_sliced] : ",bin_points_dict[bin_key][:,dim_to_be_sliced])
						median = np.median(bin_points_dict[bin_key][:,dim_to_be_sliced])
						#print("median : ",median)

						a_bin_boundary, b_bin_boundary = bin_boundary.copy(), bin_boundary.copy()
						#print("a_bin_boundary : ",a_bin_boundary)
						a_bin_boundary[dim_to_be_sliced,1] = median
						b_bin_boundary[dim_to_be_sliced,0] = median
						bin_boundaries_dict[str(split_number)+bin_key[1:]+'a'] = a_bin_boundary
						bin_boundaries_dict[str(split_number)+bin_key[1:]+'b'] = b_bin_boundary
						
						a_points, b_points = [],[] 
						for event_number in range(bin_points_dict[bin_key].shape[0]):
							if bin_points_dict[bin_key][event_number,dim_to_be_sliced] < median: a_points.append(bin_points_dict[bin_key][event_number,:].tolist())
							else: b_points.append(bin_points_dict[bin_key][event_number,:].tolist())
						
						bin_points_dict[str(split_number)+bin_key[1:]+'a'] = np.array(a_points)
						bin_points_dict[str(split_number)+bin_key[1:]+'b'] = np.array(b_points)
						#If a bin contains no particles it should be deleted
						if len(a_points)==0: 
							del bin_points_dict[str(split_number)+bin_key[1:]+'a']
							del bin_boundaries_dict[str(split_number)+bin_key[1:]+'a']
                                                if len(b_points)==0: 
                                                        del bin_points_dict[str(split_number)+bin_key[1:]+'b']
                                                        del bin_boundaries_dict[str(split_number)+bin_key[1:]+'b']


			check_key = '3aaa'
			#check_key = str(max_number_of_splits)
			#for i in range(max_number_of_splits): check_key += random.choice(['a','b'])
			#print("bin_boundaries_dict.keys() : ", bin_boundaries_dict.keys())
			#print("bin_boundaries_dict[check_key] : ",bin_boundaries_dict[check_key])
			#print("bin_points_dict.keys(): ", bin_points_dict.keys()) 
			#print("bin_points_dict[check_key][0,:] : ",bin_points_dict[check_key][0,:])
				
			bins_sample01_dict= {}
			signed_Scp2_dict= {}

			for number_of_splits in number_of_splits_list:
				print("\nnumber_of_splits : ",number_of_splits)
				bins_sample0, bins_sample1 = [] , [] 
				for bin_key, bin_points in bin_points_dict.items():
					if str(number_of_splits) in bin_key:
						labels_in_bin = bin_points[:,-1]
						#print("labels_in_bin : ",labels_in_bin)
						bin_sample0 = np.count_nonzero( labels_in_bin == 0)
						bin_sample1 = np.count_nonzero( labels_in_bin == 1)
						print("systematics_fraction : ",systematics_fraction,"\nbin_sample0 : ",bin_sample0) 
						#simulate uncertainties 
						if(bin_sample0!=0):bin_sample0 += int(round(np.random.normal(0.,systematics_fraction*bin_sample0)))
                                                if(bin_sample1!=0):bin_sample1 += int(round(np.random.normal(0.,systematics_fraction*bin_sample1)))
						bins_sample01_dict[bin_key]=[bin_sample0,bin_sample1]
						signed_Scp2_dict[bin_key] = np.square(float(bin_sample1-bin_sample0))/(float(bin_sample1)+float(bin_sample0)+np.square(float(bin_sample1)*systematics_fraction)+np.square(float(bin_sample1)*systematics_fraction))*np.sign(bin_sample1-bin_sample0)
						#print("\n\nbin_sample0 : ",bin_sample0, "\n bins_sample0 : ", bins_sample0 )
						#print("type(bin_sample0) : ",type(bin_sample0), "  type(bins_sample0) : ",type(bins_sample0))
						bins_sample0.append(bin_sample0)
						#print("  bins_sample0 : ", bins_sample0, "\n\n" )
						bins_sample1.append(bin_sample1)
				bins_sample0, bins_sample1 = np.array(bins_sample0,dtype=float), np.array(bins_sample1, dtype=float)
				print("bins_sample0 : ",bins_sample0,"\n bins_sample1 : ",bins_sample1)
				#element wise subtraction and division
				Scp2 = ((bins_sample1-bins_sample0)**2)/ (bins_sample1+bins_sample0+(systematics_fraction*bins_sample1)**2+(systematics_fraction*bins_sample0)**2 )
				#Scp2 =  np.divide(np.square(np.subtract(bins_sample1,bins_sample0)),np.add(bins_sample1,bins_sample0))
				if DEBUG:
					print(Scp2)

				#nansum ignores all the contributions that are Not A Number (NAN)
				Chi2 = np.nansum(Scp2)
				if DEBUG:
					print("Chi2")
					print(Chi2)
				dof=bins_sample0.shape[0]-1
			
				pvalue= 1 - stats.chi2.cdf(Chi2,dof)
				
				print("\nThe p value for Scp2 = ",Scp2," and Chi2 = ", Chi2, " is ",pvalue,"\n\n")
				if DEBUG:
					print(bins_sample0)
					print(bins_sample1)
					print("Chi2/dof : {0}".format(str(Chi2/dof)))

					print("pvalue : {0}".format(str(pvalue)))
				score_dict[str(number_of_splits)].append(pvalue)

		for number_of_splits in number_of_splits_list:
			name = orig_name + "_" +str(dim_data) + "D_" + str(number_of_splits) + "_splits"
			print("score_dict[{}] : ".format(number_of_splits), score_dict[str(number_of_splits)]) 
			with open(name+"_adaptive_binning_p_values", "wb") as test_statistics_file:
				for score in score_dict[str(number_of_splits)]:
					test_statistics_file.write(str(score)+"\n")

			

			if X_values.shape[1]==2: adaptive_binning_plot(bin_boundaries_dict,signed_Scp2_dict,number_of_splits,X_values,name)
			histo_plot_pvalue(score_dict[str(number_of_splits)],50,"p value","Frequency","p value distribution "+ str(number_of_splits) + " splits",name+"_adaptive_binning")








import numpy as np
import matplotlib
from matplotlib.pylab import *
import matplotlib.pyplot as plt
from matplotlib.patches import Rectangle
import matplotlib.cm as cm
import time


def histo_plot_pvalue(U_0,abins,axlabel,aylabel,atitle,aname):
        bins_probability=np.histogram(U_0,bins=abins)[1]

        #Finding the p values corresponding to 1,2 and 3 sigma significance.
        no_one_std_dev=sum(i < (1-0.6827) for i in U_0)
        no_two_std_dev=sum(i < (1-0.9545) for i in U_0)
        no_three_std_dev=sum(i < (1-0.9973) for i in U_0)

        print(no_one_std_dev,no_two_std_dev,no_three_std_dev)

        with open(aname+"_p_values_1_2_3_std_dev.txt",'w') as p_value_1_2_3_std_dev_file:
                p_value_1_2_3_std_dev_file.write(str(no_one_std_dev)+'\t'+str(no_two_std_dev)+'\t'+str(no_three_std_dev)+'\n')

        #plt.rc('text', usetex=True)
        textstr = '$1\sigma=%i$\n$2\sigma=%i$\n$3\sigma=%i$'%(no_one_std_dev, no_two_std_dev, no_three_std_dev)


        # Making a histogram of the probability predictions of the algorithm. 
        fig_pred_0= plt.figure()
        ax1_pred_0= fig_pred_0.add_subplot(1, 1, 1)
        n0, bins0, patches0 = ax1_pred_0.hist(U_0, bins=bins_probability, facecolor='red', alpha=0.5)
        ax1_pred_0.set_xlabel(axlabel)
        ax1_pred_0.set_ylabel(aylabel)
        ax1_pred_0.set_title(atitle)
        plt.xlim([0,1])

        # these are matplotlib.patch.Patch properties
        props = dict(boxstyle='round', facecolor='wheat', alpha=0.5)

        # place a text box in upper left in axes coords
        ax1_pred_0.text(0.85, 0.95, textstr, transform=ax1_pred_0.transAxes, fontsize=14,
        verticalalignment='top', bbox=props)

        fig_pred_0.savefig(aname+"_p_values_plot.png")
        #plt.close(fig_pred_0)

def adaptive_binning_plot(bin_boundaries_dict,signed_Scp2_dict,number_of_splits,X_values,name):
	if X_values.shape[1]==2:
		fig= plt.figure()
		ax1= fig.add_subplot(1, 1, 1)	
		signed_Scp2_values = []
		for bin_key, bin_Scp2 in signed_Scp2_dict.items(): 
			if str(number_of_splits) in bin_key: 
				signed_Scp2_values.append(bin_Scp2)	
		norm = matplotlib.colors.Normalize(min(signed_Scp2_values),max(signed_Scp2_values))
		my_cmap = cm.get_cmap('Greys')
		
		ax1.scatter(X_values[:,0], X_values[:,1], c='black',s=5, lw = 0)

		for bin_key, bin_boundary in bin_boundaries_dict.items():
			if str(number_of_splits) in bin_key:
				#print("Boundaries : ",bin_boundary[0,0], bin_boundary[1,0], bin_boundary[0,1],bin_boundary[1,1])
				cd=ax1.add_patch(Rectangle((bin_boundary[0,0], bin_boundary[1,0]),bin_boundary[0,1]-bin_boundary[0,0] , bin_boundary[1,1]-bin_boundary[1,0],alpha=0.5, edgecolor="black",facecolor = my_cmap(norm(signed_Scp2_dict[bin_key])),linewidth=2))
	
		plt.xlim(0.0,1.0)
		plt.ylim(0.0,1.0)	
		fig.savefig(name+"_bin_definitions.png")
		print("The plot {}_bin_definitions.png has been made".format(name))
	else:
		print("You can only make this plot in 2D")






import sys 
from chi2_plots import *

"""
This script can be used to get the p value for the Miranda method (=chi squared). It takes input files with column vectors corresponding to 
features and lables. 
"""

print(__doc__)
import sys
sys.path.insert(0,'../..')
import os
from scipy import stats
import numpy as np
import matplotlib.pyplot as plt 
import numpy.matlib
from matplotlib.colors import Normalize

from sklearn.preprocessing import StandardScaler

##############################################################################
# Setting parameters
#
#orig_name= sys.argv[1]
#single_no_bins_list_list= ast.literal_eval(sys.argv[2])
#dim_list = ast.literal_eval(sys.argv[3])
#comp_file_list_list = ast.literal_eval(sys.argv[4])

def chi2_regular_binning(orig_name, dim_list, comp_file_list_list,single_no_bins_list):

	sample1_name="original"
	sample2_name="modified"

	shuffling_seed = 100
	DEBUG = True


	##############################################################################
	for dim_index, dim_data in enumerate(dim_list):
		print("We are now in "+str(dim_data) + " Dimensions")
		#comp_file_list=[]
		comp_file_list = comp_file_list_list[dim_index]
		#for i in range(100):
			#comp_file_list.append((os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/higher_dimensional_gauss/gauss_data/data_high" +str(dim_data)+"Dgauss_10000_0.5_0.1_0.0_{0}.txt".format(i),os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/higher_dimensional_gauss/gauss_data/data_high"+str(dim_data)+"Dgauss_10000_0.5_0.1_0.01_{0}.txt".format(i))) 
			#comp_file_list.append((os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/legendre/legendre_data/data_legendre_contrib0__1__10__sample_{0}.txt".format(i),os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/legendre/legendre_data/data_legendre_contrib0__1__9__sample_{0}.txt".format(i)))
			#comp_file_list.append((os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/gaussian_same_projection_on_each_axis/gauss_data/gaussian_same_projection_on_each_axis_redefined_{1}D_1000_0.6_0.2_0.1_{0}.txt".format(i,dim_data),os.environ['MLToolsDir']+"/Dalitz/gaussian_samples/gaussian_same_projection_on_each_axis/gauss_data/gaussian_same_projection_on_each_axis_redefined_{1}D_1000_0.6_0.2_0.075_{0}.txt".format(i,dim_data)))
		print(comp_file_list)

		for single_no_bins in single_no_bins_list:
			score_list=[]
			for comp_file_0,comp_file_1 in comp_file_list:
				name = orig_name + "_" +str(dim_data) + "D_" + str(single_no_bins)
				print("single_no_bins : ",single_no_bins)
				print("Operating of files :"+comp_file_0+"   "+comp_file_1)

				#extracts data from the files
				features_0=np.loadtxt(comp_file_0,dtype='d')
				features_1=np.loadtxt(comp_file_1,dtype='d')

				#determine how many data points are in each sample
				no_0=features_0.shape[0]
				no_1=features_1.shape[0]

				#Give all samples in file 0 the label 0 and in file 1 the feature 1
				label_0=np.zeros((no_0,1))
				label_1=np.ones((no_1,1))

				#Create an array containing samples and features.
				data_0=np.c_[features_0,label_0]
				data_1=np.c_[features_1,label_1]

				data=np.r_[data_0,data_1]

				no_dim = data.shape[1]-1
				no_bins = [single_no_bins]*no_dim       

				np.random.shuffle(data)

				labels=data[:,-1]

				X_values= data[:,:-1]
				X_max   = np.amax(data,axis=0)[:-1] 
				X_min   = np.amin(data,axis=0)[:-1]
				X_width = (np.divide(np.subtract(X_max,X_min),no_bins))
				#print(X_width)


				setup_command_0 = "bins_sample0=np.zeros(("
				setup_command_1 = "bins_sample1=np.zeros(("
				for dim in range(no_dim):
					setup_command_0 += str(int(no_bins[dim]))+","
					setup_command_1 += str(int(no_bins[dim]))+","
				setup_command_0=setup_command_0[:-1]+"))"
				setup_command_1=setup_command_1[:-1]+"))"
				exec setup_command_0
				exec setup_command_1

				for i in range(no_0+no_1):
					#bin position
					#x_bin=int(np.floor((Xx_values[i]-Xx_min)/Xx_width))
					#y_bin=int(np.floor((Xy_values[i]-Xy_min)/Xy_width))

					pos_bins=np.floor(np.divide(np.subtract(X_values[i,:],X_min[:]),X_width[:]))
					#print(pos_bins)

					#eliminate boundary effects
					for dim in range(no_dim):
						if(pos_bins[dim]==no_bins[dim]):
							pos_bins[dim] -=1

					#if(pos_bins[0]==no_bins[0]):
						#pos_bins[0] -=1


					bin_command_0 = "bins_sample0["
					bin_command_1 = "bins_sample1["
					for dim in range(no_dim):
						bin_command_0 += str(int(pos_bins[dim]))+","
						bin_command_1 += str(int(pos_bins[dim]))+","
					bin_command_0=bin_command_0[:-1]+"]"
					bin_command_1=bin_command_1[:-1]+"]"

					#print("labels[i]: {0}".format(str(int(labels[i]))))
					#print(bin_command_0)
					if(labels[i]==0):
						#print(bin_command_0)
						#bins_sample0[y_bin,x_bin] +=1
						exec bin_command_0 + "+=1"
						#eval(bin_command_0)
						#print("labels[i]: {0}".format(str(int(labels[i]))))


					else:
						#bins_sample1[y_bin,x_bin] +=1
						exec bin_command_1 + "+=1"
						#print("labels[i]: {0}".format(str(int(labels[i]))))
				if DEBUG:
					print(bins_sample0)
					print(np.sum(bins_sample0))

					print(bins_sample1)
					print(np.sum(bins_sample1))
				#element wise subtraction and division
				Scp2 =  np.divide(np.square(np.subtract(bins_sample1,bins_sample0)),np.add(bins_sample1,bins_sample0))
				if DEBUG:
					print(Scp2)

				#nansum ignores all the contributions that are Not A Number (NAN)
				Chi2 = np.nansum(Scp2)
				if DEBUG:
					print("Chi2")
					print(Chi2)
				dof=no_bins[0]
				for dim in range(1,no_dim):
					dof *= no_bins[1]
				dof-=1
			
				pvalue= 1 - stats.chi2.cdf(Chi2,dof)

				if DEBUG:
					print(bins_sample0)
					print(bins_sample1)
					print("Chi2/dof : {0}".format(str(Chi2/dof)))

					print("pvalue : {0}".format(str(pvalue)))
				score_list.append(pvalue)

			with open(name+"_bins_p_values", "wb") as test_statistics_file:
				for score in score_list:
					test_statistics_file.write(str(score)+"\n")

			histo_plot_pvalue(score_list,50,"p value","Frequency","p value distribution "+ str(single_no_bins) + " bins",name+"_bins")









from setuptools import setup

setup(name='adaptive_binning_chisquared_2sam',
      version='0.5',
      description='This repository contains a 2 sample chi squared test the uses adaptive binning using the approach of Roederer et al. (http://onlinelibrary.wiley.com/doi/10.1002/1097-0320(20010901)45:1%3C47::AID-CYTO1143%3E3.0.CO;2-A/epdf)',
      url='https://github.com/weissercn/adaptive_binning_chisquared_2sam',
      author='Constantin Weisser',
      author_email='weissercn@gmail.com',
      license='MIT',
      packages=['adaptive_binning_chisquared_2sam'],
      install_requires=[
          'numpy',
          'scipy'
      ],
      zip_safe=False)



import sys
import os
os.environ["adaptive_binning_chisquared_2sam"]=os.getcwd()
#sys.path.insert(0, './GoF')
import chi2_regular_binning, chi2_adaptive_binning, chi2_adaptive_binning_systematics , chi2_plots, calling_chi2_example
